package Hilbert {
  import flash.display.CapsStyle;
  import flash.display.Graphics;
  import flash.geom.Rectangle;

  /**
   * This class contains drawing methods for the Hilbert Curve.
   */
  public class HilbertCurve {
    /**
     * Paints the Hilbert curve.
     *
     * @param g Graphic context to draw in
     * @param rect Rectangle to fill
     * @param order Hilbert curve order
     * @param color Color to use
     */
    public static function draw(g:Graphics, order:uint,
          rect:Rectangle, color:uint):void {
      var thickness:Number = Math.min(rect.width, rect.height) / (3 << order);

      g.clear();
      g.lineStyle(thickness, color, 1, false, "normal",
          CapsStyle.ROUND);

      g.moveTo(thickness/2, rect.height-thickness/2);
      drawRec(g, order, rect, 1, 3);
      g.lineTo(rect.width-thickness/2, rect.height-thickness/2);
    }

    /* Fills rectangle with a segment of Hilbert curve recursively.
     *
     * @param g Graphic context to draw in
     * @param order Hilbert curve order
     * @param rect Rectangle to fill
     * @param start Number of the corner the curve starts in
     * @param end Number of the corner the curve ends in
     */
    private static function drawRec(g:Graphics, order:uint,
        rect:Rectangle, start:uint, end:uint):void {
      if (order == 0) {
        g.lineTo(rect.x + rect.width/2, rect.y + rect.height/2);
      } else {
        drawRec(g, order-1, quadrant(rect, start), start, ~end);
        drawRec(g, order-1, quadrant(rect, ~end), start, end);
        drawRec(g, order-1, quadrant(rect, ~start), start, end);
        drawRec(g, order-1, quadrant(rect, end), ~start, end);
      }
    }

    /* Divides a rectangle into 4 quadrants and returns
     * the n'th of them.
     *
     * @param rect Rectangle to quadrisect
     * @param n Number of quadrant to return
     * @return n'th quadrant as a Rectangle
     */
    private static function quadrant(rect:Rectangle, n:uint):Rectangle {
      var quadrantWidth:Number = rect.width/2;
      var quadrantHeight:Number = rect.height/2;

      var shiftY:uint = n & 1;
      var shiftX:uint = (n >> 1) & 1;
      return new Rectangle(
        rect.x + shiftX*quadrantWidth,
        rect.y + shiftY*quadrantWidth,
        quadrantWidth, quadrantHeight
      );
    }
  }
}

