package Hilbert {
  import flash.geom.Rectangle;
  import mx.core.UIComponent;

  /**
   * Hilbert curve plot widget.
   */
  [Style(name="size", type="Number", format="Length", inherit="yes")]
  [Style(name="color", type="uint", format="Color", inherit="yes")]
  public class HilbertCurvePlot extends UIComponent {
    // Internal variables:
    private var _order:uint = Parameters.nDefault;
        // Hilbert curve order

    /* Getter and setter for the "order" property.
     */
    public function get order():uint {return _order; }
    public function set order(v:uint):void {
      if (v != _order) {
        _order = v;
        invalidateProperties(); // This marks component to be redrawn
      }
    }

    /* This method is called whenever some style property changes.
     */
    override public function styleChanged(styleProp:String):void {
      super.styleChanged(styleProp);
      invalidateProperties();  // This marks component to be redrawn
    }

    /* This obligatory method sets UI-Component measures
     */
    protected override function measure():void {
      measuredWidth = measuredHeight = getStyle("size");
    }

    /* This method is called by environment every time
     * the properties are changed. We override it in order
     * to redraw the curve after such changes.
     */
    protected override function commitProperties():void {
        draw()
    }

    /* Draws the Hilbert curve.
     */
    private function draw():void {
      var size:Number = getStyle("size");
      var color:uint = getStyle("color");
      var square:Rectangle = new Rectangle(0, 0, size, size);
      HilbertCurve.draw(graphics, order, square, color);
    }
  }

